@@ -1,3 +1,3 @@
 from . import basic as basic
+from . import brain_inspired as brain_inspired
 # from .hybrid import *
-# from .brain_inspired import *
